# -*- coding: utf-8 -*-
'''
Created by Emre MENTESE on 24/01/2022
Coding with Python.
'''

import math

def decorator_quantity(func):
    def inner(client,symbol):
        Account = client.account()
        for coin in Account['balances']:
            if coin['asset'] != symbol:
                continue
            return func(client,symbol,coin = coin)
    return inner

def round_stepsize(value, step_size) -> float:
    precision: int = int(round(-math.log(step_size, 10), 0))
    return float(round(value, precision))

@decorator_quantity
def quantity_free(client,symbol,coin = None) -> float:
    '''
    * Get your coins free quantitiy.

    - Input
        * Coin symbol: BTC , DENT , ETH , TROY

    - Outputs
        * Free quantitiy -> Float
    '''
    return float(coin['free'])

@decorator_quantity
def quantity_locked(client,symbol,coin = None) -> float:
    '''
    * Get your coins locked quantitiy.

    - Input
        * Coin symbol: BTC , DENT , ETH , TROY

    - Outputs
        * Locked quantitiy -> Float
    '''
    return float(coin['locked'])

@decorator_quantity
def quantity_all(client,symbol,coin = None) -> float:
    '''
    * Get your coins all (locked + free) quantitiy.

    - Input
        * Coin symbol: BTC , DENT , ETH , TROY

    - Outputs
        * all quantitiy -> Float
    '''
    return float(coin['locked']) + float(coin['free'])

@decorator_quantity
def balance_free(client,symbol,coin = None) -> float:
    '''
    * Get your coins free balance $ (Instant Price USDT).

    - Input
        * Coin symbol: BTC , DENT , ETH , TROY

    - Outputs
        * free balance -> Float
    '''
    ticker_price = float(client.ticker_price(symbol + "USDT")['price'])
    balance =  float(coin['free']) * ticker_price
    return balance

@decorator_quantity
def balance_locked(client,symbol,coin = None) -> float:
    '''
    * Get your coins locked balance $ (Instant Price USDT).

    - Input
        * Coin symbol: BTC , DENT , ETH , TROY

    - Outputs
        * locked balance -> Float
    '''
    ticker_price = float(client.ticker_price(symbol + "USDT")['price'])
    balance =  float(coin['locked']) * ticker_price
    return balance

@decorator_quantity
def balance_all(client,symbol,coin = None) -> float:
    '''
    * Get your coins all (locked + free) balance $ (Instant Price USDT).

    - Input
        * Coin symbol: BTC , DENT , ETH , TROY

    - Outputs
        * all balance -> Float
    '''
    ticker_price = float(client.ticker_price(symbol + "USDT")['price'])
    balance =  (float(coin['locked']) + float(coin['free'])) * ticker_price
    return balance

def balance_usdt(client) -> float:
    '''
    * Get your account balance USDT (Instant Price USDT $)
    '''
    hesap = client.account()
    for coin in hesap['balances']:
        if coin['asset'] != 'USDT':
            continue
        return float(coin['free'])

def price(client,symbol) -> float:
    '''
    * This function return the instant price value the symbol.

    - Input
        * Coin market: BTCUSDT

    - Outputs
        * instant price value  -> Float
    '''
    return float(client.ticker_price(symbol)['price'])

def price_before_24hr(client,symbol) -> float:
    '''
        * This function return the before 24hr price value the symbol.

        - Input
        * Coin market: BTCUSDT

        - Outputs
        * before 24hr price -> Float
        '''
    return float(client.ticker_24hr(symbol)['openPrice'])

def price_change24(client,symbol) -> float:
    """
    * Get 24hr Ticker Price Change value

    - Input
        * Coin market: BTCUSDT
    
    - Outputs
        * 24hr price change -> Float
    """
    return float(client.ticker_24hr(symbol)['priceChange'])

def price_high24(client,symbol) -> float:
    """
    * Get 24hr Ticker Price High value

    - Input
        * Coin market: BTCUSDT
    
    - Outputs
        * 24hr High price -> Float
    """
    return float(client.ticker_24hr(symbol)['highPrice'])

def price_low24(client,symbol) -> float:
    """
    * Get 24hr Ticker Price Low value

    - Input
        * Coin market: BTCUSDT
    
    - Outputs
        * 24hr Low price -> Float
    """
    return float(client.ticker_24hr(symbol)['lowPrice'])

def price_change_percent24(client,symbol) -> float:
    """
    * Get 24hr Ticker Price Change Percent (%)

    - Input
        * Coin market: BTCUSDT

    - Outputs
        * 24hr Price Change Percent -> Float
    """
    return float(client.ticker_24hr(symbol)['priceChangePercent'])

def candlesticks(client,symbol,time,count) -> list:
    """
    * Get candlestick data for the given symbol graph.

    - time : 1m, 15m 1h , 4h , 1d , 1w ...
    """
    return client.klines(symbol, time, count)

def market_buy_with_price(client,market,price) -> dict:
    """
    * New order to buy coin with MARKET price (Use to price).

    For Example: BTCUSDT -> market_buy_with_price(client,BTCUSDT,USDT,150) You will get $150 BTC

    - Input
        * market --> BTCUSDT , BTCBNB, BTCETH... (str)
        * price -->  560 $ , 0.323 BNB, 22 ETH... (float)

    - Outputs
        * if success to order return order info dict
        {'buy_price': '372.90000000', 'qty': '0.00100000', 'comission': '0.00000075'}

    """
    try:
        # TRADE RULES CONTROL

        r = client.exchange_info(market)
        quoteAsset = r['symbols']['quoteAsset']

        # 1- PRICE_FILTER Control
        minPrice = float(r['symbols'][0]['filters'][0]['minPrice'])
        maxPrice = float(r['symbols'][0]['filters'][0]['maxPrice'])
        tickSize = float(r['symbols'][0]['filters'][0]['tickSize'])
        validated_price = round_stepsize(price,tickSize)

        if (validated_price < minPrice) or (price > maxPrice):
            raise Exception(f"PRICE_FILTER Error: Price is not in range Min:{minPrice} - Max:{maxPrice}")

        # 2- MIN_NOTIONAL Control
        minNotional = float(r['symbols'][0]['filters'][3]['minNotional'])
        if validated_price < minNotional:
            raise Exception(f"MIN_NOTIONAL Control Error: Minimum Trade Amount: {minNotional} {quoteAsset}")

        # 3- Balance Control
        balance = quantity_free(client,quoteAsset)
        if balance < validated_price:
            raise Exception(f"Balance Control Error: You don't have enough balance to buy this coin ({quoteAsset}:{balance})")

        params = {
            "symbol": market,
            "side": "BUY",
            "type": "MARKET",
            "quoteOrderQty":str(validated_price),
        }
        response = client.new_order(**params)
        return {"buy_price":response['fills'][0]['price'],"qty":response['fills'][0]['qty'],"comission":response['fills'][0]['commission']}
        
    except Exception as e:
        print(e)
        return False
        
def market_buy_with_quantity(client,market,quantity) -> dict:
    """
    * New order to buy coin with MARKET price (Use to quantity).

    For Example: BTCUSDT -> market_buy_with_price(client,BTCUSDT,USDT,0.01234) You will get 0.01234 BTC

    - Input
        * market --> BTCUSDT , BTCBNB, BTCETH...
        * symbol --> USDT , BNB , ETH...
        * quantity -->  560 , 0.323, 22...

    - Outputs
        * if success to order return order info dict
        {'buy_price': '372.90000000', 'qty': '0.00100000', 'comission': '0.00000075'}
    """

    try:
        # TRADE RULES CONTROL

        # Quantity type control
        if not isinstance(quantity,(int,float)):
            raise Exception("Quantity Error: Quantity must be float or int.")
        params = {
            "symbol": market,
            "side": "BUY",
            "type": "MARKET",
            "quantity":str(quantity),
        }
        response = client.new_order(**params)
        return {"buy_price":response['fills'][0]['price'],"qty":response['fills'][0]['qty'],"comission":response['fills'][0]['commission']}
    except Exception as e:
        print(e)
        return False




# -*- coding: utf-8 -*-
'''
Created by Emre MENTESE on 24/01/2022
Coding with Python.
'''
from binancema.coininfo import price
class indicator:

    def __init__(self,client):
        self.client = client

    def SMA(self,series,length) -> float:
        '''
        * Simple Moving Average
        * Referance (01/11/2021): https://www.tradingview.com/pine-script-reference/#fun_sma
        * If you want look at the original SMA function coded with Pine Script, visit the reference.

        - length --> Data count len(series) (int)
        - series --> data (list) or (int- float)
        '''
        if isinstance(length,int):
            pass
        else:
            raise Exception("SMA Calculating Error: Length must be integer.")

        if isinstance(series,list):
            sum = 0 
            for i in series:
                if isinstance(i,(int,float)):
                    sum += i
                else:
                    raise Exception("SMA Calculating Error: series elements must be integer or float.")
            return sum

        elif (series,(float,int)):
            return series / length

        else:
            raise Exception("SMA Calculating Error: series must be integer & float or list.")

        
    def EMA(self,close,length,previous_ema) -> float:
        '''
        * Exponential Moving Avarage 
        * Referance (01/11/2021): https://www.tradingview.com/pine-script-reference/#fun_ema 
        * If you want look at the original EMA function coded with Pine Script, visit the reference.

            Firstly Binance not give EMA information for cripto coins. If you want calculate to last EMA, you must use
        previous EMA. You can't take a previous EMA with Binance. You must calculate. You need previous to previous EMA
        to calculate previous EMA. This goes as far as the first EMA. If you want calculate to the first EMA, you must 
        use SMA in the EMA calculate. However, calculating first EMA using SMA is inconvenient and takes too long and 
        Binance has an api limit. Binance max give to the 1000 candlestick bars. You can't calculate SMA with this candlestick bars.
        
        * For example: You can't calculate BTC-USDT  5 minute grahpic's EMA. Because you need more candlestick bars information (Billion)
        * That's Why we can add one more input in the function. (previous_EMA)
        * This input can be any EMA provided it is an EMA before the EMA of the last candlestick bars.

        length : Number of candlestick bars (Integer)
        previous_ema : can be any EMA provided it is an EMA before the EMA of the last candlestick bars. (Float-Integer)
        close  : The closing value of the candlestick bars after the previous EMA candlestick bar (Float-Integer)

        return --> next EMA

        '''
        if isinstance(length,int):
            pass
        else:
            raise Exception("EMA Calculating Error: Length must be integer.")

        if isinstance(previous_ema,(float,int)):
            pass
        else:
            raise Exception("EMA Calculating Error: previous_ema must be integer or float.")

        if isinstance(close,(float,int)):
            pass
        else:
            raise Exception("EMA Calculating Error: close must be integer or float.")

        alpha = 2 / (length + 1)
        ema = (close * alpha) + (previous_ema * (1-alpha))
        return ema


    def coins_instant_ema(self,length,previous_ema,symbol) -> float:
        '''
        * This function return instant ema using with the ema of the previous candlestick bars

        Note:
            * When the graphic move to a next candlestick bars you have to change the ema and run it again !
            * This function can't calculating next candlestick bars's ema. İt can just return instant (dynamic) ema !

        Examples
        - symbol = "BTCUSDT"
        - length = 9
        - previous_ema (9) = 62017.40
        
        '''

        close_value = price(self.client,symbol)
        return self.EMA(close_value,length,previous_ema)


    def MACD(self,close,fast,slow,signal,previous_ema_fast,previous_ema_slow,previous_macd):
        '''
        * Moving Average Convergence Divergence
        * Referance (01/11/2021): https://en.tradingview.com/ideas/macd/
        * If you want look at the original MACD function coded with Pine Script, visit the reference.
        

        - close  --> The closing value of the candlestick bars after the previous EMA candlestick bar (Float-Integer)
        - fast   --> fast ema length 
        - slow   --> slow ema length 
        - signal --> signal length

        - previous_ema_fast --> can be any ema provided it is an ema before the ema of the last candlestick bars. (Float-Integer)
        - previous_ema_slow --> can be any ema provided it is an ema before the ema of the last candlestick bars. (Float-Integer)
        - previous_macd     --> can be any macd provided it is an macd before the macd of the last candlestick bars. (Float-Integer)

        '''
        fastMA = self.EMA(close,fast,previous_ema_fast) # next fast ema
        slowMA = self.EMA(close,slow,previous_ema_slow) # next slow ema
        macd = fastMA - slowMA  # macd value 
        signal_value = self.EMA(macd,signal,previous_macd) # signal value
        return [macd,signal_value]

    def coins_instant_macd(self,symbol,fast,slow,signal,previous_ema_fast,previous_ema_slow,previous_macd):
        '''
        * This function return instant macd using with the macd of the previous candlestick bars

        Note:
            * When the graphic move to a next candlestick bars you have to change the macd and run it again !
            * This function can't calculating next candlestick bars's macd. İt can just return instant macd !

        Examples
        - symbol = "BTCUSDT" string
        - fast = 12 int
        - slow = 26 int
        - signal = 9 int
        - previous_ema_fast 
        - previous_ema_slow
        - previous_macd = 62017.40
        '''
        close = price(self.client,symbol)
        return self.MACD(close,fast,slow,signal,previous_ema_fast,previous_ema_slow,previous_macd)


from setuptools import setup, find_packages

about = {}

with open("README.md", "r") as readme_file:
    readme = readme_file.read()

NAME = "binance-ema"
DESCRIPTION = ("Get crypto coin informations & calculate the custom or constant indicators. (WITH BINANCE API)")
AUTHOR = "Emre MENTESE"
URL = "https://github.com/emrementese/binance-ema"
VERSION = "0.1.7"
URLS = {
  'MyWebsite': 'http://www.emrementese.com/',
  'Github': 'https://github.com/emrementese',
  'Source': 'https://github.com/emrementese/binance-ema',
  'Download': 'https://pypi.org/project/binance-ema/#files',
}

setup(
    name=NAME,
    version=VERSION,
    license="MIT",
    description=DESCRIPTION,
    long_description=readme,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email= "emrmentese@gmail.com",
    project_urls = URLS,
    url=URL,
    keywords=["Binance", "Public API","EMA","MACD","TRADE","Crypto","Coin","Indicators","BTC","ETH","USDT"],
    install_requires=[
        "binance-connector==1.10.0",
    ],
    packages=find_packages('src'),
    package_dir={'':'src'},
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Financial and Insurance Industry",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    python_requires=">=3.6",
)
